/*  SNAPMONO - Snapshot mono display buffer to the clipboard
 *
 *  Copyright 1992, Cherry Hill Software
 *  All Rights Reserved.
 *
 *  COMMENTS
 *	This program illustrates the use of the SetSelectorBase()
 *	and SetSelectorLimit() functions and accesses hardware.
 *
 *  WARNING
 *	This program assumes that a Monochrome Display Adapter (MDA)
 *	is installed.
 */

#include "../include/winflags.h"
#undef NOKERNEL
#undef NOMEMMGR
#undef NOUSER
#undef NOCLIPBOARD
#undef NOMB
#include <windows.h>

#include "../include/winkrnl.h"

#define MDA_ADDRESS 0x000B0000
#define NUM_ROWS 25
#define NUM_COLS 80

int snapmono(void)
{
    LPSTR pText ;	    /* Pointer to snapshot text buffer	*/
    LPSTR pVideo ;	    /* Pointer to video hardware	*/
    WORD sel ;		    /* Selector that we will build	*/
    int i, line, lastcol, col ;
    char ch ;
    HGLOBAL hText ;	    /* Handle to pass to clipboard	*/

    /* Allocate a data selector (use Stack Seg as template) */
    sel = AllocSelector(HIWORD((LONG)(LPSTR)&sel)) ;

    /* Set the linear base address and segment limit of the selector */
    SetSelectorBase(sel, MDA_ADDRESS) ;
    SetSelectorLimit(sel, NUM_ROWS*NUM_COLS*2-1) ;
    pVideo = (LPSTR)MAKELONG(0,sel) ;	/* Turn it into a pointer */

    /* Allocate our snapshot buffer; allow CR/LF at end of each line */
    pText = (LPSTR) GlobalLock(GlobalAlloc(GMEM_MOVEABLE,(NUM_ROWS+2)*NUM_COLS+1)) ;

    if (pText)		/* If successfully allocated */
	{
	i = 0 ;
	for (line=0;line<NUM_ROWS;line++)     /* Through each display line */
	    {
	    lastcol = i ;
	    for (col=0;col<NUM_COLS;col++)    /* Through each column */
		{
		ch = pVideo[(line*NUM_COLS+col)*2] ;	/* Get the character */
		pText[i++] = ch ;
		if (ch != 0x00 && ch != ' ')
		    lastcol = i ;	/* Strip trailing blanks */
		}
	    i = lastcol ;
	    pText[i++] = '\r' ; 	/* append CR/LF */
	    pText[i++] = '\n' ;
	    }
	pText[i++] = '\0' ;		/* Tag a NUL on the end */
	FreeSelector(sel) ;		/*     cf. clipboard format */

	hText = (HGLOBAL) GlobalHandle(HIWORD(pText)) ;
	GlobalUnlock(hText) ;			    /* Unlock the buffer */
	GlobalReAlloc(hText, i, GMEM_MOVEABLE) ;    /* Change its size */

	/* NOTE: If we wanted, we could translate the extended codes
	 * to Windows-ASCII with the OEMToAnsi() function.  Instead, we
	 * will be copying this directly to a PC-8 character set printer,
	 * so we leave the characters alone.  We don't use CF_OEMTEXT
	 * because if we did, Windows would just garble it, anyway.
	 */

	/* Move the text to the clipboard */
	OpenClipboard(GetActiveWindow()) ;
	EmptyClipboard() ;
	SetClipboardData(CF_TEXT, hText) ; /* hText is no longer valid */
	CloseClipboard() ;

	return 1 ;	/* SUCCESS */
	}
    else
	MessageBox(0, "Cannot allocate snapshot buffer", "Snap Mono",0) ;
    return 0 ;
}


int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    hInstance	    = hInstance ;
    hPrevInstance   = hPrevInstance ;
    lpCmdLine	    = lpCmdLine ;
    nCmdShow	    = nCmdShow ;

    while (MessageBox(0, "Press OK to snap the Mono Display",
			 "Snap Mono",MB_OKCANCEL) == IDOK)
	{
	if (snapmono())
	    MessageBox(0, "Mono Display Successfully Snapped", "Snap Mono",0) ;
	}
    return 0 ;
}
